package main

import "fmt"
import _ "math"
import "strings"
import "strconv"
import "regexp"

type Statino struct {
	Nominativo string
	Matricola string
	Esami []int   // I voti sono fra 18 e 30; si usa 32 per indicare il 30 e lode
}

func Media(voti []int) float64 {
	sum := 0.0
	for _, voto := range voti {
		if voto == 32 {
			sum += 30.0
		} else {
			sum += float64(voto)
		}
	}
	return sum / float64(len(voti))
}

/* Data una riga nel formato

   matricola\tnominativo\tvoto,voto,voto,voto 

   restituisce lo statino corrispondente, usa il comma-ok idiom */
func Line2StatinoOld(row string) (statino Statino, ok bool) {
	parti := strings.Split(row, "\t")
	if len(parti) != 3 {
		return   // Errore
	}
	statino.Matricola = parti[0]
	statino.Nominativo = parti[1]
	stringaVoti := parti[2]
	voti := strings.Split(stringaVoti, ",")
	for _, stringaVoto := range voti {
		voto, err := strconv.Atoi(stringaVoto)
		if err != nil {
			return // Errore di conversione
		}
		if voto < 18 || (voto > 30 && voto != 32) {
			return // Errore di valore
		}
		statino.Esami = append(statino.Esami, voto)
	}
	ok = true
	return
}


func Line2Statino(row string) (statino Statino, ok bool) {
	matricoleRE := regexp.MustCompile("^([0-9]|[A-Z])+") // Sequenza di uno o più cifre o lettere maiuscole a inizio stringa
	nomeRE := regexp.MustCompile("([A-Z]|[a-z]|_| )+") // Sequenza di uno o più lettere, underscore, spazi
	numeroRE := regexp.MustCompile("([0-9])+")     // Numero = sequenza di una o più cifre decimali

	//fmt.Printf("Sto parsando <%s>\n", row)

	// Cerco la matricola
	statino.Matricola = matricoleRE.FindString(row)
	if statino.Matricola == "" {
		return 
	}

	//fmt.Printf("\tho parsato la matricola <%s>\n", statino.Matricola)

	// Cerco il nome
	if len(row) == len(statino.Matricola) {
		return
	}
	statino.Nominativo = nomeRE.FindString(row[len(statino.Matricola)+1:])
	if statino.Nominativo == "" {
		return 
	}

	//fmt.Printf("\tho parsato la nominativo <%s>\n", statino.Nominativo)

	// Cerco e converto i voti	
	listaVoti := numeroRE.FindAllString(row[len(statino.Matricola)+1:], -1)
	for _, stringaVoto := range listaVoti {
		voto, err := strconv.Atoi(stringaVoto)
		if err != nil {
			fmt.Println("Can't happen!!!!!")
			return // Errore di conversione
		}
		if voto < 18 || (voto > 30 && voto != 32) {
			return // Errore di valore
		}
		statino.Esami = append(statino.Esami, voto)
	}

	//fmt.Printf("\tho parsato i voti <%v>\n", statino.Esami)

	ok = true
	return 
}


func main() {
	s := "31254 afdv 3 345231 yy667 fe 312"
	fmt.Println(s)
	Line2Statino(s)
}

